@@ -1,7 +1,39 @@
+/*
+ * Copyright (c) 1990 - 1994, Julianne Frances Haugh
+ * Copyright (c) 1996 - 2000, Marek Michałkiewicz
+ * Copyright (c) 2001       , Michał Moskal
+ * Copyright (c) 2003 - 2005, Tomasz Kłoczko
+ * Copyright (c) 2007 - 2009, Nicolas François
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The name of the copyright holders or contributors may not be used to
+ *    endorse or promote products derived from this software without
+ *    specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+ * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
+ * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
 
 #include <config.h>
 
-#ident "$Id: pwio.c 1752 2008-02-03 16:50:14Z nekral-guest $"
+#ident "$Id: pwio.c 2783 2009-04-23 21:19:02Z nekral-guest $"
 
 #include "prototypes.h"
 #include "defines.h"
@@ -10,23 +42,18 @@
 #include "commonio.h"
 #include "pwio.h"
 
-static void *passwd_dup (const void *ent)
+static /*@null@*/ /*@only@*/void *passwd_dup (const void *ent)
 {
 	const struct passwd *pw = ent;
 
 	return __pw_dup (pw);
 }
 
-static void passwd_free (void *ent)
+static void passwd_free (/*@out@*/ /*@only@*/void *ent)
 {
 	struct passwd *pw = ent;
 
-	free (pw->pw_name);
-	free (pw->pw_passwd);
-	free (pw->pw_gecos);
-	free (pw->pw_dir);
-	free (pw->pw_shell);
-	free (pw);
+	pw_free (pw);
 }
 
 static const char *passwd_getname (const void *ent)
@@ -70,17 +97,22 @@ static struct commonio_db passwd_db = {
 	NULL,			/* head */
 	NULL,			/* tail */
 	NULL,			/* cursor */
-	0,			/* changed */
-	0,			/* isopen */
-	0,			/* locked */
-	0			/* readonly */
+	false,			/* changed */
+	false,			/* isopen */
+	false,			/* locked */
+	false			/* readonly */
 };
 
-int pw_name (const char *filename)
+int pw_setdbname (const char *filename)
 {
 	return commonio_setname (&passwd_db, filename);
 }
 
+/*@observer@*/const char *pw_dbname (void)
+{
+	return passwd_db.filename;
+}
+
 int pw_lock (void)
 {
 	return commonio_lock (&passwd_db);
@@ -91,12 +123,12 @@ int pw_open (int mode)
 	return commonio_open (&passwd_db, mode);
 }
 
-const struct passwd *pw_locate (const char *name)
+/*@observer@*/ /*@null@*/const struct passwd *pw_locate (const char *name)
 {
 	return commonio_locate (&passwd_db, name);
 }
 
-const struct passwd *pw_locate_uid (uid_t uid)
+/*@observer@*/ /*@null@*/const struct passwd *pw_locate_uid (uid_t uid)
 {
 	const struct passwd *pwd;
 
@@ -123,7 +155,7 @@ int pw_rewind (void)
 	return commonio_rewind (&passwd_db);
 }
 
-const struct passwd *pw_next (void)
+/*@observer@*/ /*@null@*/const struct passwd *pw_next (void)
 {
 	return commonio_next (&passwd_db);
 }
@@ -138,7 +170,7 @@ int pw_unlock (void)
 	return commonio_unlock (&passwd_db);
 }
 
-struct commonio_entry *__pw_get_head (void)
+/*@null@*/struct commonio_entry *__pw_get_head (void)
 {
 	return passwd_db.head;
 }
