@@ -1,5 +1,8 @@
 /*
- * Copyright 1989 - 1994, Julianne Frances Haugh
+ * Copyright (c) 1989 - 1994, Julianne Frances Haugh
+ * Copyright (c) 1996 - 2000, Marek Michałkiewicz
+ * Copyright (c) 2000 - 2006, Tomasz Kłoczko
+ * Copyright (c) 2007 - 2009, Nicolas François
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -10,26 +13,26 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of Julianne F. Haugh nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
+ * 3. The name of the copyright holders or contributors may not be used to
+ *    endorse or promote products derived from this software without
+ *    specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY JULIE HAUGH AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL JULIE HAUGH OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+ * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
+ * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <config.h>
 
-#ident "$Id: chage.c 1751 2008-02-03 16:28:03Z nekral-guest $"
+#ident "$Id: chage.c 2851 2009-04-30 21:39:38Z nekral-guest $"
 
 #include <ctype.h>
 #include <fcntl.h>
@@ -39,64 +42,64 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <time.h>
+#ifdef ACCT_TOOLS_SETUID
 #ifdef USE_PAM
 #include "pam_defs.h"
 #endif				/* USE_PAM */
+#endif				/* ACCT_TOOLS_SETUID */
 #include <pwd.h>
 #ifdef WITH_SELINUX
 #include <selinux/selinux.h>
 #include <selinux/av_permissions.h>
 #endif
-#include "exitcodes.h"
 #include "prototypes.h"
 #include "defines.h"
 #include "pwio.h"
 #include "shadowio.h"
+/*@-exitarg@*/
+#include "exitcodes.h"
+
 /*
  * Global variables
  */
-static char *Prog;
+char *Prog;
 
-static int
- dflg = 0,			/* set last password change date */
-    Eflg = 0,			/* set account expiration date */
-    Iflg = 0,			/* set password inactive after expiration */
-    lflg = 0,			/* show account aging information */
-    mflg = 0,			/* set minimum number of days before password change */
-    Mflg = 0,			/* set maximum number of days before password change */
-    Wflg = 0;			/* set expiration warning days */
-static int amroot = 0;
+static bool
+    dflg = false,		/* set last password change date */
+    Eflg = false,		/* set account expiration date */
+    Iflg = false,		/* set password inactive after expiration */
+    lflg = false,		/* show account aging information */
+    mflg = false,		/* set minimum number of days before password change */
+    Mflg = false,		/* set maximum number of days before password change */
+    Wflg = false;		/* set expiration warning days */
+static bool amroot = false;
 
-static int pw_locked = 0;	/* Indicate if the password file is locked */
-static int spw_locked = 0;	/* Indicate if the shadow file is locked */
+static bool pw_locked  = false;	/* Indicate if the password file is locked */
+static bool spw_locked = false;	/* Indicate if the shadow file is locked */
 /* The name and UID of the user being worked on */
 static char user_name[BUFSIZ] = "";
 static uid_t user_uid = -1;
 
 static long mindays;
 static long maxdays;
-static long lastday;
+static long lstchgdate;
 static long warndays;
 static long inactdays;
-static long expdays;
-
-#ifdef USE_PAM
-static pam_handle_t *pamh = NULL;
-#endif
+static long expdate;
 
 #define	EPOCH		"1969-12-31"
 
 /* local function prototypes */
-static int isnum (const char *s);
+static bool isnum (const char *s);
 static void usage (void);
-static void date_to_str (char *, size_t, time_t);
+static void date_to_str (char *buf, size_t maxsize, time_t date);
 static int new_fields (void);
-static void print_date (time_t);
+static void print_date (time_t date);
 static void list_fields (void);
 static void process_flags (int argc, char **argv);
 static void check_flags (int argc, int opt_index);
 static void check_perms (void);
-static void open_files (int readonly);
+static void open_files (bool readonly);
 static void close_files (void);
 static void fail_exit (int code);
 
@@ -106,27 +109,26 @@ static void fail_exit (int code);
 static void fail_exit (int code)
 {
 	if (spw_locked) {
-		spw_unlock ();
+		if (spw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, spw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", spw_dbname ()));
+			/* continue */
+		}
 	}
 	if (pw_locked) {
-		pw_unlock ();
+		if (pw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
+			/* continue */
+		}
 	}
 	closelog ();
 
 #ifdef WITH_AUDIT
 	if (E_SUCCESS != code) {
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "change age",
-		              user_name, user_uid, 0);
-	}
-#endif
-
-#ifdef USE_PAM
-	if (NULL != pamh) {
-		/* If there is a PAM error, pam_end will be called by the
-		 * caller.
-		 * We always end the pam transaction with PAM_SUCCESS here.
-		 */
-		pam_end (pamh, PAM_SUCCESS);
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "change age",
+		              user_name, (unsigned int) user_uid, 0);
 	}
 #endif
 
@@ -136,15 +138,15 @@ static void fail_exit (int code)
 /*
  * isnum - determine whether or not a string is a number
  */
-static int isnum (const char *s)
+static bool isnum (const char *s)
 {
 	while ('\0' != *s) {
 		if (!isdigit (*s)) {
-			return 0;
+			return false;
 		}
 		s++;
 	}
-	return 1;
+	return true;
 }
 
 /*
@@ -155,7 +157,7 @@ static void usage (void)
 	fputs (_("Usage: chage [options] [LOGIN]\n"
 	         "\n"
 	         "Options:\n"
-	         "  -d, --lastday LAST_DAY        set last password change to LAST_DAY\n"
+	         "  -d, --lastday LAST_DAY        set date of last password change to LAST_DAY\n"
 	         "  -E, --expiredate EXPIRE_DATE  set account expiration date to EXPIRE_DATE\n"
 	         "  -h, --help                    display this help message and exit\n"
 	         "  -I, --inactive INACTIVE       set password inactive after expiration\n"
@@ -176,10 +178,10 @@ static void date_to_str (char *buf, size_t maxsize, time_t date)
 
 	tp = gmtime (&date);
 #ifdef HAVE_STRFTIME
-	strftime (buf, maxsize, "%Y-%m-%d", tp);
+	(void) strftime (buf, maxsize, "%Y-%m-%d", tp);
 #else
-	snprintf (buf, maxsize, "%04d-%02d-%02d",
-	          tp->tm_year + 1900, tp->tm_mon + 1, tp->tm_mday);
+	(void) snprintf (buf, maxsize, "%04d-%02d-%02d",
+	                 tp->tm_year + 1900, tp->tm_mon + 1, tp->tm_mday);
 #endif				/* HAVE_STRFTIME */
 }
 
@@ -195,66 +197,61 @@ static void date_to_str (char *buf, size_t maxsize, time_t date)
 static int new_fields (void)
 {
 	char buf[200];
-	char *cp;
 
-	puts (_("Enter the new value, or press ENTER for the default"));
-	puts ("");
+	(void) puts (_("Enter the new value, or press ENTER for the default"));
+	(void) puts ("");
 
 	snprintf (buf, sizeof buf, "%ld", mindays);
 	change_field (buf, sizeof buf, _("Minimum Password Age"));
-	mindays = strtol (buf, &cp, 10);
-	if (   ((mindays == 0) && ('\0' != *cp))
+	if (   (getlong (buf, &mindays) == 0)
 	    || (mindays < -1)) {
 		return 0;
 	}
 
 	snprintf (buf, sizeof buf, "%ld", maxdays);
 	change_field (buf, sizeof buf, _("Maximum Password Age"));
-	maxdays = strtol (buf, &cp, 10);
-	if (   ((maxdays == 0) && ('\0' != *cp))
+	if (   (getlong (buf, &maxdays) == 0)
 	    || (maxdays < -1)) {
 		return 0;
 	}
 
-	date_to_str (buf, sizeof buf, lastday * SCALE);
+	date_to_str (buf, sizeof buf, lstchgdate * SCALE);
 
 	change_field (buf, sizeof buf, _("Last Password Change (YYYY-MM-DD)"));
 
 	if (strcmp (buf, EPOCH) == 0) {
-		lastday = -1;
+		lstchgdate = -1;
 	} else {
-		lastday = strtoday (buf);
-		if (lastday == -1) {
+		lstchgdate = strtoday (buf);
+		if (lstchgdate == -1) {
 			return 0;
 		}
 	}
 
 	snprintf (buf, sizeof buf, "%ld", warndays);
 	change_field (buf, sizeof buf, _("Password Expiration Warning"));
-	warndays = strtol (buf, &cp, 10);
-	if (   ((warndays == 0) && ('\0' != *cp))
+	if (   (getlong (buf, &warndays) == 0)
 	    || (warndays < -1)) {
 		return 0;
 	}
 
 	snprintf (buf, sizeof buf, "%ld", inactdays);
 	change_field (buf, sizeof buf, _("Password Inactive"));
-	inactdays = strtol (buf, &cp, 10);
-	if (   ((inactdays == 0) && ('\0' != *cp))
+	if (   (getlong (buf, &inactdays) == 0)
 	    || (inactdays < -1)) {
 		return 0;
 	}
 
-	date_to_str (buf, sizeof buf, expdays * SCALE);
+	date_to_str (buf, sizeof buf, expdate * SCALE);
 
 	change_field (buf, sizeof buf,
 	              _("Account Expiration Date (YYYY-MM-DD)"));
 
 	if (strcmp (buf, EPOCH) == 0) {
-		expdays = -1;
+		expdate = -1;
 	} else {
-		expdays = strtoday (buf);
-		if (expdays == -1) {
+		expdate = strtoday (buf);
+		if (expdate == -1) {
 			return 0;
 		}
 	}
@@ -269,15 +266,25 @@ static void print_date (time_t date)
 	char buf[80];
 
 	tp = gmtime (&date);
-	strftime (buf, sizeof buf, "%b %d, %Y", tp);
-	puts (buf);
+	if (NULL == tp) {
+		(void) printf ("time_t: %lu\n", date);
+	} else {
+		(void) strftime (buf, sizeof buf, "%b %d, %Y", tp);
+		(void) puts (buf);
+	}
 #else
 	struct tm *tp;
-	char *cp;
+	char *cp = NULL;
 
 	tp = gmtime (&date);
-	cp = asctime (tp);
-	printf ("%6.6s, %4.4s\n", cp + 4, cp + 20);
+	if (NULL != tp) {
+		cp = asctime (tp);
+	}
+	if (NULL != cp) {
+		(void) printf ("%6.6s, %4.4s\n", cp + 4, cp + 20);
+	} else {
+		(void) printf ("time_t: %lu\n", date);
+	}
 #endif
 }
 
@@ -297,27 +304,30 @@ static void list_fields (void)
 	 * The "last change" date is either "never" or the date the password
 	 * was last modified. The date is the number of days since 1/1/1970.
 	 */
-	fputs (_("Last password change\t\t\t\t\t: "), stdout);
-	if (lastday < 0) {
-		puts (_("never"));
-	} else if (lastday == 0) {
-		puts (_("password must be changed"));
+	(void) fputs (_("Last password change\t\t\t\t\t: "), stdout);
+	if (lstchgdate < 0) {
+		(void) puts (_("never"));
+	} else if (lstchgdate == 0) {
+		(void) puts (_("password must be changed"));
 	} else {
-		changed = lastday * SCALE;
-		print_date (changed);
+		changed = lstchgdate * SCALE;
+		print_date ((time_t) changed);
 	}
 
 	/*
 	 * The password expiration date is determined from the last change
 	 * date plus the number of days the password is valid for.
 	 */
-	fputs (_("Password expires\t\t\t\t\t: "), stdout);
-	if ((lastday <= 0) || (maxdays >= (10000 * (DAY / SCALE)))
-	    || (maxdays < 0)) {
-		puts (_("never"));
+	(void) fputs (_("Password expires\t\t\t\t\t: "), stdout);
+	if (lstchgdate == 0) {
+		(void) puts (_("password must be changed"));
+	} else if (   (lstchgdate < 0)
+	           || (maxdays >= (10000 * (DAY / SCALE)))
+	           || (maxdays < 0)) {
+		(void) puts (_("never"));
 	} else {
 		expires = changed + maxdays * SCALE;
-		print_date (expires);
+		print_date ((time_t) expires);
 	}
 
 	/*
@@ -326,25 +336,29 @@ static void list_fields (void)
 	 * number of inactive days is added. The resulting date is when the
 	 * active will be disabled.
 	 */
-	fputs (_("Password inactive\t\t\t\t\t: "), stdout);
-	if ((lastday <= 0) || (inactdays < 0) ||
-	    (maxdays >= (10000 * (DAY / SCALE))) || (maxdays < 0)) {
-		puts (_("never"));
+	(void) fputs (_("Password inactive\t\t\t\t\t: "), stdout);
+	if (lstchgdate == 0) {
+		(void) puts (_("password must be changed"));
+	} else if (   (lstchgdate < 0)
+	           || (inactdays < 0)
+	           || (maxdays >= (10000 * (DAY / SCALE)))
+	           || (maxdays < 0)) {
+		(void) puts (_("never"));
 	} else {
 		expires = changed + (maxdays + inactdays) * SCALE;
-		print_date (expires);
+		print_date ((time_t) expires);
 	}
 
 	/*
 	 * The account will expire on the given date regardless of the
 	 * password expiring or not.
 	 */
-	fputs (_("Account expires\t\t\t\t\t\t: "), stdout);
-	if (expdays < 0) {
-		puts (_("never"));
+	(void) fputs (_("Account expires\t\t\t\t\t\t: "), stdout);
+	if (expdate < 0) {
+		(void) puts (_("never"));
 	} else {
-		expires = expdays * SCALE;
-		print_date (expires);
+		expires = expdate * SCALE;
+		print_date ((time_t) expires);
 	}
 
 	/*
@@ -391,42 +405,74 @@ static void process_flags (int argc, char **argv)
 			     &option_index)) != -1) {
 		switch (c) {
 		case 'd':
-			dflg++;
+			dflg = true;
 			if (!isnum (optarg)) {
-				lastday = strtoday (optarg);
-			} else {
-				lastday = strtol (optarg, 0, 10);
+				lstchgdate = strtoday (optarg);
+			} else if (   (getlong (optarg, &lstchgdate) == 0)
+			           || (lstchgdate < -1)) {
+				fprintf (stderr,
+				         _("%s: invalid date '%s'\n"),
+				         Prog, optarg);
+				usage ();
 			}
 			break;
 		case 'E':
-			Eflg++;
+			Eflg = true;
 			if (!isnum (optarg)) {
-				expdays = strtoday (optarg);
-			} else {
-				expdays = strtol (optarg, 0, 10);
+				expdate = strtoday (optarg);
+			} else if (   (getlong (optarg, &expdate) == 0)
+			           || (expdate < -1)) {
+				fprintf (stderr,
+				         _("%s: invalid date '%s'\n"),
+				         Prog, optarg);
+				usage ();
 			}
 			break;
 		case 'h':
 			usage ();
 			break;
 		case 'I':
-			Iflg++;
-			inactdays = strtol (optarg, 0, 10);
+			Iflg = true;
+			if (   (getlong (optarg, &inactdays) == 0)
+			    || (inactdays < -1)) {
+				fprintf (stderr,
+				         _("%s: invalid numeric argument '%s'\n"),
+				         Prog, optarg);
+				usage ();
+			}
 			break;
 		case 'l':
-			lflg++;
+			lflg = true;
 			break;
 		case 'm':
-			mflg++;
-			mindays = strtol (optarg, 0, 10);
+			mflg = true;
+			if (   (getlong (optarg, &mindays) == 0)
+			    || (mindays < -1)) {
+				fprintf (stderr,
+				         _("%s: invalid numeric argument '%s'\n"),
+				         Prog, optarg);
+				usage ();
+			}
 			break;
 		case 'M':
-			Mflg++;
-			maxdays = strtol (optarg, 0, 10);
+			Mflg = true;
+			if (   (getlong (optarg, &maxdays) == 0)
+			    || (maxdays < -1)) {
+				fprintf (stderr,
+				         _("%s: invalid numeric argument '%s'\n"),
+				         Prog, optarg);
+				usage ();
+			}
 			break;
 		case 'W':
-			Wflg++;
-			warndays = strtol (optarg, 0, 10);
+			Wflg = true;
+			if (   (getlong (optarg, &warndays) == 0)
+			    || (warndays < -1)) {
+				fprintf (stderr,
+				         _("%s: invalid numeric argument '%s'\n"),
+				         Prog, optarg);
+				usage ();
+			}
 			break;
 		default:
 			usage ();
@@ -476,10 +522,13 @@ static void check_flags (int argc, int opt_index)
  */
 static void check_perms (void)
 {
+#ifdef ACCT_TOOLS_SETUID
 #ifdef USE_PAM
+	pam_handle_t *pamh = NULL;
 	struct passwd *pampw;
 	int retval;
-#endif
+#endif				/* USE_PAM */
+#endif				/* ACCT_TOOLS_SETUID */
 
 	/*
 	 * An unprivileged user can ask for their own aging information, but
@@ -492,38 +541,35 @@ static void check_perms (void)
 		fail_exit (E_NOPERM);
 	}
 
+#ifdef ACCT_TOOLS_SETUID
 #ifdef USE_PAM
-	retval = PAM_SUCCESS;
-
 	pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
-	if (pampw == NULL) {
-		retval = PAM_USER_UNKNOWN;
+	if (NULL == pampw) {
+		fprintf (stderr,
+		         _("%s: Cannot determine your user name.\n"),
+		         Prog);
+		exit (E_NOPERM);
 	}
 
-	if (retval == PAM_SUCCESS) {
-		retval = pam_start ("chage", pampw->pw_name, &conv, &pamh);
-	}
+	retval = pam_start ("chage", pampw->pw_name, &conv, &pamh);
 
-	if (retval == PAM_SUCCESS) {
+	if (PAM_SUCCESS == retval) {
 		retval = pam_authenticate (pamh, 0);
-		if (retval != PAM_SUCCESS) {
-			pam_end (pamh, retval);
-		}
 	}
 
-	if (retval == PAM_SUCCESS) {
+	if (PAM_SUCCESS == retval) {
 		retval = pam_acct_mgmt (pamh, 0);
-		if (retval != PAM_SUCCESS) {
-			pam_end (pamh, retval);
-		}
 	}
 
-	if (retval != PAM_SUCCESS) {
+	if (NULL != pamh) {
+		(void) pam_end (pamh, retval);
+	}
+	if (PAM_SUCCESS != retval) {
 		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
-		pamh = NULL;
 		fail_exit (E_NOPERM);
 	}
 #endif				/* USE_PAM */
+#endif				/* ACCT_TOOLS_SETUID */
 }
 
 /*
@@ -532,25 +578,25 @@ static void check_perms (void)
  *	In read-only mode, the databases are not locked and are opened
  *	only for reading.
  */
-static void open_files (int readonly)
+static void open_files (bool readonly)
 {
 	/*
 	 * Lock and open the password file. This loads all of the password
 	 * file entries into memory. Then we get a pointer to the password
 	 * file entry for the requested user.
 	 */
-	if (!readonly && (pw_lock () == 0)) {
-		fprintf (stderr,
-		         _("%s: can't lock password file\n"), Prog);
-		SYSLOG ((LOG_ERR, "failed locking %s", PASSWD_FILE));
-		fail_exit (E_NOPERM);
-	}
 	if (!readonly) {
-		pw_locked = 1;
+		if (pw_lock () == 0) {
+			fprintf (stderr,
+			         _("%s: cannot lock %s; try again later.\n"),
+			         Prog, pw_dbname ());
+			fail_exit (E_NOPERM);
+		}
+		pw_locked = true;
 	}
 	if (pw_open (readonly ? O_RDONLY: O_RDWR) == 0) {
-		fprintf (stderr, _("%s: can't open password file\n"), Prog);
-		SYSLOG ((LOG_ERR, "failed opening %s", PASSWD_FILE));
+		fprintf (stderr, _("%s: cannot open %s\n"), Prog, pw_dbname ());
+		SYSLOG ((LOG_WARN, "cannot open %s", pw_dbname ()));
 		fail_exit (E_NOPERM);
 	}
 
@@ -560,19 +606,19 @@ static void open_files (int readonly)
 	 * does not have to exist in this case; a new entry will be created
 	 * for this user if one does not exist already.
 	 */
-	if (!readonly && (spw_lock () == 0)) {
-		fprintf (stderr,
-		         _("%s: can't lock shadow password file\n"), Prog);
-		SYSLOG ((LOG_ERR, "failed locking %s", SHADOW_FILE));
-		fail_exit (E_NOPERM);
-	}
 	if (!readonly) {
-		spw_locked = 1;
+		if (spw_lock () == 0) {
+			fprintf (stderr,
+			         _("%s: cannot lock %s; try again later.\n"),
+			         Prog, spw_dbname ());
+			fail_exit (E_NOPERM);
+		}
+		spw_locked = true;
 	}
 	if (spw_open (readonly ? O_RDONLY: O_RDWR) == 0) {
 		fprintf (stderr,
-		         _("%s: can't open shadow password file\n"), Prog);
-		SYSLOG ((LOG_ERR, "failed opening %s", SHADOW_FILE));
+		         _("%s: cannot open %s\n"), Prog, spw_dbname ());
+		SYSLOG ((LOG_WARN, "cannot open %s", spw_dbname ()));
 		fail_exit (E_NOPERM);
 	}
 }
@@ -588,8 +634,8 @@ static void close_files (void)
 	 */
 	if (spw_close () == 0) {
 		fprintf (stderr,
-		         _("%s: can't rewrite shadow password file\n"), Prog);
-		SYSLOG ((LOG_ERR, "failed rewriting %s", SHADOW_FILE));
+		         _("%s: failure while writing changes to %s\n"), Prog, spw_dbname ());
+		SYSLOG ((LOG_ERR, "failure while writing changes to %s", spw_dbname ()));
 		fail_exit (E_NOPERM);
 	}
 
@@ -598,14 +644,22 @@ static void close_files (void)
 	 * will be re-written.
 	 */
 	if (pw_close () == 0) {
-		fprintf (stderr, _("%s: can't rewrite password file\n"), Prog);
-		SYSLOG ((LOG_ERR, "failed rewriting %s", PASSWD_FILE));
+		fprintf (stderr, _("%s: failure while writing changes to %s\n"), Prog, pw_dbname ());
+		SYSLOG ((LOG_ERR, "failure while writing changes to %s", pw_dbname ()));
 		fail_exit (E_NOPERM);
 	}
-	spw_unlock ();
-	spw_locked = 0;
-	pw_unlock ();
-	pw_locked = 0;
+	if (spw_unlock () == 0) {
+		fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, spw_dbname ());
+		SYSLOG ((LOG_ERR, "failed to unlock %s", spw_dbname ()));
+		/* continue */
+	}
+	spw_locked = false;
+	if (pw_unlock () == 0) {
+		fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
+		SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
+		/* continue */
+	}
+	pw_locked = false;
 }
 
 /*
@@ -628,13 +682,12 @@ static void update_age (const struct spwd *sp, const struct passwd *pw)
 		memzero (&spwent, sizeof spwent);
 		spwent.sp_namp = xstrdup (pw->pw_name);
 		spwent.sp_pwdp = xstrdup (pw->pw_passwd);
-		spwent.sp_flag = -1;
+		spwent.sp_flag = SHADOW_SP_FLAG_UNSET;
 
 		pwent.pw_passwd = SHADOW_PASSWD_STRING;	/* XXX warning: const */
 		if (pw_update (&pwent) == 0) {
 			fprintf (stderr,
-			         _("%s: can't update password file\n"), Prog);
-			SYSLOG ((LOG_ERR, "failed updating %s", PASSWD_FILE));
+			         _("%s: failed to prepare the new %s entry '%s'\n"), Prog, pw_dbname (), pwent.pw_name);
 			fail_exit (E_NOPERM);
 		}
 	} else {
@@ -650,15 +703,14 @@ static void update_age (const struct spwd *sp, const struct passwd *pw)
 	 */
 	spwent.sp_max = maxdays;
 	spwent.sp_min = mindays;
-	spwent.sp_lstchg = lastday;
+	spwent.sp_lstchg = lstchgdate;
 	spwent.sp_warn = warndays;
 	spwent.sp_inact = inactdays;
-	spwent.sp_expire = expdays;
+	spwent.sp_expire = expdate;
 
 	if (spw_update (&spwent) == 0) {
 		fprintf (stderr,
-		         _("%s: can't update shadow password file\n"), Prog);
-		SYSLOG ((LOG_ERR, "failed updating %s", SHADOW_FILE));
+		         _("%s: failed to prepare the new %s entry '%s'\n"), Prog, spw_dbname (), spwent.sp_namp);
 		fail_exit (E_NOPERM);
 	}
 
@@ -681,7 +733,7 @@ static void get_defaults (const struct spwd *sp)
 			mindays = sp->sp_min;
 		}
 		if (!dflg) {
-			lastday = sp->sp_lstchg;
+			lstchgdate = sp->sp_lstchg;
 		}
 		if (!Wflg) {
 			warndays = sp->sp_warn;
@@ -690,7 +742,7 @@ static void get_defaults (const struct spwd *sp)
 			inactdays = sp->sp_inact;
 		}
 		if (!Eflg) {
-			expdays = sp->sp_expire;
+			expdate = sp->sp_expire;
 		}
 	} else {
 		/*
@@ -704,7 +756,7 @@ static void get_defaults (const struct spwd *sp)
 			mindays = -1;
 		}
 		if (!dflg) {
-			lastday = 0;
+			lstchgdate = -1;
 		}
 		if (!Wflg) {
 			warndays = -1;
@@ -713,7 +765,7 @@ static void get_defaults (const struct spwd *sp)
 			inactdays = -1;
 		}
 		if (!Eflg) {
-			expdays = -1;
+			expdate = -1;
 		}
 	}
 }
@@ -750,9 +802,9 @@ int main (int argc, char **argv)
 	audit_help_open ();
 #endif
 	sanitize_env ();
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
+	(void) setlocale (LC_ALL, "");
+	(void) bindtextdomain (PACKAGE, LOCALEDIR);
+	(void) textdomain (PACKAGE);
 
 	ruid = getuid ();
 	rgid = getgid ();
@@ -778,14 +830,15 @@ int main (int argc, char **argv)
 		fprintf (stderr,
 		         _("%s: the shadow password file is not present\n"),
 		         Prog);
-		SYSLOG ((LOG_ERR, "can't find the shadow password file"));
+		SYSLOG ((LOG_WARN, "can't find the shadow password file"));
 		closelog ();
 		exit (E_SHADOW_NOTFOUND);
 	}
 
 	open_files (lflg);
 	/* Drop privileges */
-	if (lflg && (setregid (rgid, rgid) || setreuid (ruid, ruid))) {
+	if (lflg && (   (setregid (rgid, rgid) != 0)
+	             || (setreuid (ruid, ruid) != 0))) {
 		fprintf (stderr, _("%s: failed to drop privileges (%s)\n"),
 		         Prog, strerror (errno));
 		fail_exit (E_NOPERM);
@@ -793,8 +846,8 @@ int main (int argc, char **argv)
 
 	pw = pw_locate (argv[optind]);
 	if (NULL == pw) {
-		fprintf (stderr, _("%s: unknown user %s\n"), Prog,
-		         argv[optind]);
+		fprintf (stderr, _("%s: user '%s' does not exist in %s\n"),
+		         Prog, argv[optind], pw_dbname ());
 		closelog ();
 		exit (E_NOPERM);
 	}
@@ -815,8 +868,9 @@ int main (int argc, char **argv)
 			fail_exit (E_NOPERM);
 		}
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "display aging info",
-		              user_name, user_uid, 1);
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "display aging info",
+		              user_name, (unsigned int) user_uid, 1);
 #endif
 		list_fields ();
 		fail_exit (E_SUCCESS);
@@ -838,40 +892,40 @@ int main (int argc, char **argv)
 		else {
 			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
 			              "change all aging information",
-			              user_name, user_uid, 1);
+			              user_name, (unsigned int) user_uid, 1);
 		}
 #endif
 	} else {
 #ifdef WITH_AUDIT
 		if (Mflg) {
 			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "change max age", user_name,
-			              user_uid, 1);
+			              "change max age",
+			              user_name, (unsigned int) user_uid, 1);
 		}
 		if (mflg) {
 			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "change min age", user_name,
-			              user_uid, 1);
+			              "change min age",
+			              user_name, (unsigned int) user_uid, 1);
 		}
 		if (dflg) {
 			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "change last change date", user_name,
-			              user_uid, 1);
+			              "change last change date",
+			              user_name, (unsigned int) user_uid, 1);
 		}
 		if (Wflg) {
 			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "change passwd warning", user_name,
-			              user_uid, 1);
+			              "change passwd warning",
+			              user_name, (unsigned int) user_uid, 1);
 		}
 		if (Iflg) {
 			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "change inactive days", user_name,
-			              user_uid, 1);
+			              "change inactive days",
+			              user_name, (unsigned int) user_uid, 1);
 		}
 		if (Eflg) {
 			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "change passwd expiration", user_name,
-			              user_uid, 1);
+			              "change passwd expiration",
+			              user_name, (unsigned int) user_uid, 1);
 		}
 #endif
 	}
@@ -882,10 +936,6 @@ int main (int argc, char **argv)
 
 	SYSLOG ((LOG_INFO, "changed password expiry for %s", user_name));
 
-#ifdef USE_PAM
-	pam_end (pamh, PAM_SUCCESS);
-#endif				/* USE_PAM */
-
 	closelog ();
 	exit (E_SUCCESS);
 }
